##############################################################################
#    Copyright (C) 2001-2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

ADD_CONTENT_PERMISSION = 'Add portal content'

#Globals
import Globals
import zLOG
import string
import sys, traceback
import os.path

#Zope
from DateTime.DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Acquisition import aq_inner, aq_base
from OFS.Image import File
from App.Common import package_home

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.utils import getToolByName, tuplize
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression

from utils import getUniqueId
import MediaPermissions

factory_type_information = ({'id': 'Media Event',
                             'meta_type': 'Media Manager Event',
                             'description': 'Use live events to create new streaming events',
                             'product': 'MyMediaManager',
                             'icon': 'mediaevent_icon.gif',
                             'factory': 'addMediaEvent',
                             'filter_content_types': 0,
                             'immediate_view': 'mediaevent_edit_form',
                             'actions':({'id': 'view',
                                         'name': 'View',
                                         'action': 'mediaevent_view',
                                         'permissions': ('View',),
                                         'category': 'object'},
                                        {'id': 'edit',
                                         'name': 'Edit',
                                         'action': 'mediaevent_edit_form',
                                         'permissions': ('Manage properties',),
                                         'category': 'object'},
                                        {'id': 'agenda',
                                         'name': 'Agenda',
                                         'action': 'agenda_suite',
                                         'permissions': ('Manage properties',),
                                         'category': 'object'},
                                        {'id': 'editsuite',
                                         'name': 'Edit Suite',
                                         'action': 'edit_suite',
                                         'permissions': ('Manage properties',),
                                         'category': 'object'},
                            )},)

_actions = [ ActionInformation(
                id='mediaevent_live'
            , title='Live'
            , action=Expression(
                    text='string: ${folder_url}/mediaevent_live')
            , condition=Expression(
                    text='folder/portal_medianet/hasLivePlugin')
            , permissions=(MediaPermissions.StreamLive,)
            , category='object'
            , visible=1
            ),
        ]

def _dateStrings(when):
    strings = {}
    if when == 'None':
        when = None
    if when:
       if type(when) == type(''):
           when = DateTime(when)
       strings['year'] = str(when.year())
       strings['month'] = str(when.month())
       strings['day'] = str(when.day())
    else:
       strings['year'] = ''
       strings['month'] = ''
       strings['day'] = ''
    return strings

class EventRessource(File):
    meta_type = 'Media Event Ressource'
    security = ClassSecurityInfo()

    def __init__(self, id, title, file, filename):
        File.__init__(self, id, title, file)
        self._filename = filename

    def edit(self, data):
        """
        Edit the ressource object
        """
        title = data.get('title', None)
        if title:
            self.title = title

        fileobj = data.get('ressourcefile', '')
        if fileobj and fileobj.read(1):
            fileobj.seek(0)
            filename = fileobj.filename
            # when uploaded from a windows client
            # the filename contains the entire path
            if filename.find('\\') != -1:
                fileparts = filename.split('\\')
                filename = fileparts[len(fileparts)-1]
            self.manage_upload(fileobj)
            self._filename = filename
            msg = 'Ressource updated'

    def cb_isMoveable(self):
        return 1

    def cb_isCopyable(self):
        return 1

    def getFilename(self):
        return getattr(self, '_filename', None)

class MediaEvent(PortalFolder, PortalContent, DefaultDublinCoreImpl):
    """
        A Media Event is used to present media in your CMF Portal
    """

    meta_type='Media Manager Event'
    portal_type='Object'
    security = ClassSecurityInfo()
    EVENT_PIC_ID = 'eventpic'
    DEF_PIC_SRC = 'mmm_eventpic.gif'
    PIC_DISP_HEIGHT = 240
    PIC_DISP_WIDTH = 320

    def __init__( self, id):
        DefaultDublinCoreImpl.__init__(self)
        self.start_date = DateTime()
        self.end_date = DateTime()
        self.id = id
        self._clip_paths = []
        self._last_encoders = []

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return actions provided by tool
        """
        return _actions

    def _initializeEvent(self):
        files = self.listFileObjects()
        if not files:
            fileid = getUniqueId()
            self.invokeFactory(id=fileid, type_name='Master File')
            self.invokeFactory(id=self.EVENT_PIC_ID, type_name='Image')
            masterfile = getattr(self, fileid)
            masterfile.edit(filename=fileid)
            medianet = getToolByName(self, 'portal_medianet')
            liveserver = medianet.getDefaultServer('live')
            archiveserver = medianet.getDefaultServer('ondemand')
            if liveserver:
                masterfile.assignServer(liveserver)
            if archiveserver:
                masterfile.assignServer(archiveserver)
            eventpic = getattr(self, self.DEF_PIC_SRC, None)
            if eventpic:
                data = {}
                try:
                    data['filedata'] = eventpic.data
                    self.editPicture(datadict=data)
                except:
                    mpath = "%s/zpt_skins" % package_home(globals())
                    picpath = os.path.join(mpath, 'mmm_eventpic.gif')
                    filedata = open(picpath)
                    data['filedata'] = filedata
                    self.editPicture(datadict=data)
                    filedata.close()

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        " Add self to the workflow and catalog. "
        # Recurse in the children (ObjectManager)
        PortalFolder.manage_afterAdd(self, item, container)
        # Add self to workflow and catalog
        PortalContent.manage_afterAdd(self, item, container)
        item._initializeEvent()

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        " Remove self from the workflow and catalog. "
        # We don't call PortalContent.manage_beforeDelete because
        #  it deals with talkbacks which are subobjects...
        # Remove self from catalog
        if aq_base(container) is not aq_base(self):
            self.unindexObject()
        # Then recurse in the children (ObjectManager)
        PortalFolder.manage_beforeDelete(self, item, container)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'indexObject')
    def indexObject(self):
        """
            Index the object in the portal catalog.
        """
        catalog = getToolByName(self, 'portal_catalog', None)
        if catalog is not None:
            catalog.indexObject(self)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'unindexObject')
    def unindexObject(self):
        """
            Unindex the object from the portal catalog.
        """
        catalog = getToolByName(self, 'portal_catalog', None)
        if catalog is not None:
            catalog.unindexObject(self)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[]):
        """
            Reindex the object in the portal catalog.
            If idxs is present, only those indexes are reindexed.
            The metadata is always updated.

            Also update the modification date of the object,
            unless specific indexes were requested.
        """
        if idxs == []:
            # Update the modification date.
            if hasattr(aq_base(self), 'notifyModified'):
                self.notifyModified()
        catalog = getToolByName(self, 'portal_catalog', None)
        if catalog is not None:
            catalog.reindexObject(self, idxs=idxs)

    def _verifyObjectPaste(self, object, validate_src=1):
            # This assists the version in OFS.CopySupport.
            # It enables the clipboard to function correctly
            # with objects created by a multi-factory.
            if (hasattr(object, '__factory_meta_type__') and
                hasattr(self, 'all_meta_types')):
                mt = object.__factory_meta_type__
                method_name=None
                permission_name = None
                meta_types = self.all_meta_types
                if callable(meta_types): meta_types = meta_types()
                for d in meta_types:
                    if d['name']==mt:
                        method_name=d['action']
                        permission_name = d.get('permission', None)
                        break

                if permission_name is not None:
                    if _checkPermission(permission_name,self):
                        if not validate_src:
                            # We don't want to check the object on the clipboard
                            return
                        try: parent = aq_parent(aq_inner(object))
                        except: parent = None
                        if getSecurityManager().validate(None, parent,
                                                         None, object):
                            # validation succeeded
                            return
                        raise 'Unauthorized', object.getId()
                    else:
                        raise 'Unauthorized', permission_name
                #
                # Old validation for objects that may not have registered
                # themselves in the proper fashion.
                #
                elif method_name is not None:
                    meth=self.unrestrictedTraverse(method_name)
                    if hasattr(meth, 'im_self'):
                        parent = meth.im_self
                    else:
                        try:    parent = aq_parent(aq_inner(meth))
                        except: parent = None
                    if getSecurityManager().validate(None, parent, None, meth):
                        # Ensure the user is allowed to access the object on the
                        # clipboard.
                        if not validate_src:
                            return
                        try: parent = aq_parent(aq_inner(object))
                        except: parent = None
                        if getSecurityManager().validate(None, parent,
                        None, object):
                            return
                        id = object.getId()
                        raise 'Unauthorized', id
                    else:
                        raise 'Unauthorized', method_name
        #PortalFolder.inheritedAttribute('_verifyObjectPaste')(self, object, validate_src)

    security.declarePrivate('_datify')
    def _datify(self, date):
        if date == 'None':
            date = None
        elif not isinstance(date, DateTime):
            if date is not None:
                date = DateTime(date)
        return date

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, REQUEST):
        """
        Edit the media event
        """
        req = REQUEST
        title = req.get('title', None)
        if title is not None:
            self.setTitle(title)
        desc = req.get('description', None)
        if desc is not None:
            self.setDescription(desc)

        eff = req.get('effective_date', None)
        if eff is not None:
            self.setEffectiveDate(eff)

        exp = req.get('expiration_date', None)
        if exp is not None:
            self.setExpirationDate(exp)

        rights = req.get('rights', None)
        if rights is not None:
            self.setRights(rights)

        lan = req.get('language', None)
        if lan is not None:
            self.setLanguage(lan)

        subject = req.get('subject', None)
        if subject is not None:
            self.setSubject(subject)

        start_day = req.get('start_date', None)
        start_month = req.get('start_month', None)
        start_year = req.get('start_year', None)
        start_hour = req.get('start_hour', None)
        start_ten = req.get('start_tenmin', None)
        start_one = req.get('start_onemin', None)
        if start_day is not None:
            start_date = "%s/%s/%s %s:%s%s" % (start_year, start_month, start_day, start_hour, start_ten, start_one)
            self._setStartDate(start_date)

        stop_day = req.get('stop_date', None)
        stop_month = req.get('stop_month', None)
        stop_year = req.get('stop_year', None)
        stop_hour = req.get('stop_hour', None)
        stop_ten = req.get('stop_tenmin', None)
        stop_one = req.get('stop_onemin', None)
        if stop_day is not None:
            stop_date = "%s/%s/%s %s:%s%s" % (stop_year, stop_month, stop_day, stop_hour, stop_ten, stop_one)
            self._setEndDate(stop_date)

        self._setAuthor(req.get('author', None))
        self.reindexObject()
        msg = 'Media Event metadata updated'
        req_url = req.get('HTTP_REFERER', None)
        if not req_url:
            req_url = '%s/mediaevent_edit_form' % self.absolute_url()
        else:
            has_query = req_url.find('?')
            if has_query != -1:
                req_url, trash = req_url.split('?')
        req.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, string.replace(msg, ' ', '+')))

    security.declarePrivate('_setStartDate')
    def _setStartDate(self, start_date=None):
        """
        Set the startdate of the event
        """
        if start_date is not None:
            self.start_date = self._datify(date=start_date)

    security.declarePrivate('_setEndDate')
    def _setEndDate(self, end_date):
        """
        Set the enddate of the event
        """
        if end_date is not None:
            self.end_date = self._datify(date=end_date)

    security.declarePrivate('_setAuthor')
    def _setAuthor(self, author=None):
        """
        Set the author of the event
        """
        if author is not None:
            self._author = author

    security.declarePublic('getAuthor')
    def getAuthor(self):
        """
        Return the author of the event
        """
        if hasattr(self, '_author'):
            return self._author
        else:
            return None

    security.declarePublic('start')
    def start(self):
        """
        Get the event start
        """
        date = None
        if hasattr(aq_inner(self), 'start_date'):
            date = self.start_date
        return date is None and self.created() or date

    security.declarePublic('end')
    def end(self):
        """
        Get the event end
        """
        date = None
        if hasattr(aq_inner(self), 'end_date'):
            date = self.end_date
        return date is None and self.start() or date

    security.declarePublic('getStartStrings')
    def getStartStrings(self):
        """"
        Return the start time as a dictionary
        """
        return _dateStrings(self.start())

    security.declarePublic('getEndStrings')
    def getEndStrings(self):
        """
        Return yhe end time as a dictionary
        """
        return _dateStrings(self.end())

    security.declarePublic('buildYears')
    def buildYears(self):
        """
        Return a list containing years
        """
        result = []
        start = (DateTime().year() - 2)
        end = (DateTime().year() + 5)
        for year in range(start, end):
            result.append(str(year))
        return result

    security.declarePublic('buildMonths')
    def buildMonths(self):
        """
        Return a list containing months
        """
        result = []
        for month in range(1, 13):
            result.append('%d' % month)
        return result
        
    security.declarePublic('buildDays')
    def buildDays(self):
        """
        Return a list containing days
        """
        result = []
        for day in range(1, 32):
            result.append('%d' % day)
        return result
        
    security.declarePublic('buildHours')
    def buildHours(self):
        """
        Return a list containing hours
        """
        result = []
        for hour in range(0, 25):
            result.append('%02d' % hour)
        return result

    security.declarePublic('buildTenMins')
    def buildTenMins(self):
        """
        Return a list containing ten-minutes
        """
        result = []
        for min in range(0, 6):
            result.append('%01d' % min)
        return result

    security.declarePublic('buildOneMin')
    def buildOneMin(self):
        """
        Return a list containing one-minutes
        """
        result = []
        for min in range(0, 60):
            result.append('%01d' % min)
        return result

    security.declareProtected(CMFCorePermissions.View, 'listFileObjects')
    def listFileObjects(self):
        """
        Return a list of file objects in this event
        """
        return self.contentValues('Media Manager Master File')
        
    def getEventClipList(self):
        return getattr(self, '_clip_paths', [])

    security.declareProtected(CMFCorePermissions.View, 'listClips')
    def listClips(self):
        """
        Return a list of clip objects in this event
        """
        objs = []
        portal = getToolByName(self, 'portal_url').getPortalObject()
        for path in self.getEventClipList():
            try:
                obj = portal.restrictedTraverse(path)
                objs.append(obj)
            except:
                zLOG.LOG('Media event - list clips', zLOG.ERROR, \
                                'Exception in appending clip path', error=sys.exc_info())
        return objs

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'addClip')
    def addClip(self, clippath=None):
        """
        Add a new clip path to the list of clip paths
        """
        clips = self.getEventClipList()
        if clippath is not None:
            clips.append(clippath)
        self._clip_paths = clips
        self._p_changed = 1

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'deleteClips')
    def deleteClips(self, ids, REQUEST=None):
        """
        Delete a list of clips.
        """
        cliplist = self.getEventClipList()
        elements = list(map(int, ids))
        elements.sort()
        keep_list = []

        i = 0
        for path in cliplist:
            if i in elements:
                try:
                    clip = self.restrictedTraverse(path)
                    self._delObject(clip.id)
                except:
                    pass
            else:
                keep_list.append(path)
            i = i + 1
        self._clip_paths = keep_list
        self._p_changed = 1

        if REQUEST:
            REQUEST.RESPONSE.redirect('%s/basic_cliplist_iframe?portal_status_message=Deleted' % self.absolute_url())

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'moveUpClips')
    def moveUpClips(self, ids=(), REQUEST=None):
        """
        Move the specified clips up one slot in our list.
        """
        elements = list( map( int, ids ) )  # Convert to a list of integers.
        elements.sort()

        cliplist = self.getEventClipList()

        for idx in elements:
            idx2 = idx - 1
            if idx2 < 0:
                # Wrap to the bottom.
                idx2 = len(cliplist) - 1
            # Swap.
            a = cliplist[idx2]
            cliplist[idx2] = cliplist[idx]
            cliplist[idx] = a

        self._clip_paths = cliplist
        self._p_changed = 1

        if REQUEST:
            REQUEST.RESPONSE.redirect('%s/basic_cliplist_iframe?portal_status_message=Moved' % self.absolute_url())

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'moveDownClips')
    def moveDownClips(self, ids=(), REQUEST=None):
        """
        Move the specified clips down one slot in our list.
        """
        elements = list( map( int, ids ) )  # Convert to a list of integers.
        elements.sort()
        elements.reverse()

        cliplist = self.getEventClipList()

        for idx in elements:
            idx2 = idx + 1
            if idx2 >= len(cliplist):
                # Wrap to the top.
                idx2 = 0
            # Swap.
            a = cliplist[idx2]
            cliplist[idx2] = cliplist[idx]
            cliplist[idx] = a

        self._clip_paths = cliplist
        self._p_changed = 1

        if REQUEST:
            REQUEST.RESPONSE.redirect('%s/basic_cliplist_iframe?portal_status_message=Moved' % self.absolute_url())

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'editPicture')
    def editPicture(self, REQUEST=None, datadict=None):
        """
        Edit the event illustration
        """
        eventpic = self.getEventPicture()
        msg = None
        if REQUEST:
            req = REQUEST
            filedata = req.get('eventpic', None)
            title = req.get('title', '')
            description = req.get('description', '')
            req_url = req.get('HTTP_REFERER', None)
            msg = 'title and description updated'
            if not req_url:
                req_url = '%s/liveevent_edit_form' % self.absolute_url()
            else:
                has_query = req_url.find('?')
                if has_query != -1:
                    req_url, trash = req_url.split('?')
        elif datadict:
            filedata = datadict.get('filedata', None)
            title = datadict.get('title', '')
            description = datadict.get('description', '')
            req_url = None
        else:
            req_url = None
            filedata = None
            title = ''
            description = ''
        if not eventpic:
            picid = self.EVENT_PIC_ID
            self.invokeFactory(id=picid, type_name='Image')
            eventpic = getattr(self, picid)

        if eventpic:
            if filedata:
                eventpic.edit(precondition='', file=filedata)
                msg = 'Picture Uploaded, %s' % msg
            #else:
                #$$Here we should upload the content of the default event picture for events
            eventpic.editMetadata(title=title, description=description)
        if msg and req_url:
            req.RESPONSE.redirect('%s?portal_status_message=%s' % (req_url, string.replace(msg, ' ', '+')))

    security.declarePublic('getEventPicture')
    def getEventPicture(self):
        """
        Return the event picture for the event.
        If no picture exists, try to add a default picture
        """
        if self.EVENT_PIC_ID in self.objectIds():
            try:
                pic = self.restrictedTraverse(self.EVENT_PIC_ID)
            except:
                zLOG.LOG('Media Event', zLOG.ERROR, 'getEventPicture', error=sys.exc_info())
                pic = None
        else:
            try:
                self.invokeFactory(id=self.EVENT_PIC_ID, type_name='Image')
                pic = self.restrictedTraverse(self.EVENT_PIC_ID)
                #$$Here we should upload the content of the default event picture for events
            except:
                zLOG.LOG('Media Event', zLOG.ERROR, 'getEventPicture', error=sys.exc_info())
                pic = None
        return pic

    security.declarePublic('getEventPicSrc')
    def getEventPicSrc(self, disp_height=None, disp_width=None):
        """
        Return the a dictionary containing links for popup and source for the image tag when inserting an event's illustration
        """
        pic = self.getEventPicture()
        dict = {}
        if pic:
            try:
                width = int(pic.width) + 35
                height = int(pic.height) + 70
            except:
                width = 320 + 35
                height = 240 + 70
            href = "javascript:openWindow('%s/eventpic_popup', 'element_image', '%d', '%d');" % (pic.absolute_url(), width, height)
            src = "%s/getEventImageTag?display_height=%s&display_width=%s" % (self.absolute_url(), disp_height, disp_width)
            dict['href'] = href
            dict['src'] = src
        else:
            dict['href'] = None
            dict['src'] = '%s/%s' % (self.absolute_url(), self.DEF_PIC_SRC)
        return dict

    security.declarePublic('getRessources')
    def getRessources(self):
        """
        Return all ressource objects for the event
        """
        return self.objectValues('Media Event Ressource')

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'addRessource')
    def addRessource(self, REQUEST=None, data=None):
        """
        Add a new ressouce object to the product
        """
        if REQUEST is not None:
            req = REQUEST
            req_url = req.get('HTTP_REFERER', None)
            if not req_url:
                req_url = "%s/mediaevent_edit_form" % self.absolute_url()
            has_q = req_url.find('?')
            if has_q != -1:
                req_url, trash = req_url.split('?')
        else:
            req = data
            req_url = None

        title = req.get('new_title', '')
        file = req.get('new_file', None)
        if file and file.read(1):
            file.seek(0)
            filename = file.filename
            if filename.find('\\') != -1:
                fileparts = filename.split('\\')
                filename = fileparts[len(fileparts)-1]
            newid = filename

        ressouce = EventRessource(id=newid, title=title, file=file, filename=filename)
        self._setObject(newid, ressouce)
        msg = 'Ressouce added'
        if req_url:
            req.RESPONSE.redirect("%s?portal_status_message=%s" % (req_url, msg.replace(' ', '+')))

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'editRessources')
    def editRessources(self, REQUEST=None, data=None):
        """
        Edit the ressource object (check for ids)
        """
        if REQUEST is not None:
            req = REQUEST
            req_url = req.get('HTTP_REFERER', None)
            if not req_url:
                req_url = "%s/mediaevent_edit_form" % self.absolute_url()
            has_q = req_url.find('?')
            if has_q != -1:
                req_url, trash = req_url.split('?')
        else:
            req = data
            req_url = None

        ressourceids = req.get('ressourceids', None)
        if ressourceids is not None:
            for ressourceid in ressourceids:
                postfix = ressourceid
                title = req.get('title_%s' % postfix, None)
                file = req.get('file_%s' % postfix, None)
                if file and file.read(1):
                    file.seek(0)
                    filename = file.filename
                    if filename.find('\\') != -1:
                        fileparts = filename.split('\\')
                        filename = fileparts[len(fileparts)-1]
                    newid = filename
                    self.manage_renameObject(postfix, newid)
                else:
                    newid = ressourceid

                ressourceobj = getattr(self, newid)
                data = {'title': title
                          , 'ressourcefile': file}
                ressourceobj.edit(data=data)

        msg = 'Ressource(s) updated'
        if req_url:
            req.RESPONSE.redirect("%s?portal_status_message=%s" % (req_url, msg.replace(' ', '+')))

    security.declarePublic('setPrefFormat')
    def setPrefFormat(self, serverinfo_id, REQUEST):
        """
        Set the preferred format to use with this event
        """
        resp = REQUEST.RESPONSE
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Format cookies are %s' % str(REQUEST.cookies))
        resp.setCookie('pref_user_format', serverinfo_id, path='/'+self.absolute_url(relative=1))
        if REQUEST.cookies.has_key('pref_user_format'):
            extra = ''
        else:
            extra = '?noCookies=true'
        if REQUEST.cookies.has_key('pref_edit_format'):
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Expire edit cookie')
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Expire edit cookie is: %s' % str(REQUEST.cookies['pref_edit_format']))
            resp.expireCookie('pref_edit_format', path='/'+self.absolute_url(relative=1))
        zLOG.LOG(self.meta_type, zLOG.INFO, 'After - format cookies are %s' % str(REQUEST.cookies))
        req_url = "%s/mediaevent_view%s" % (self.absolute_url(), extra)
        resp.redirect(req_url)

    security.declareProtected(CMFCorePermissions.View, 'validPrefFormat')
    def validPrefFormat(self):
        """
        Returns true if the user has a valid preferred format
        """
        req = self.REQUEST
        userpref = req.cookies.get('pref_user_format',None)
        if userpref:
            try:
                serverinfo = self.restrictedTraverse(userpref)
            except:
                serverinfo = None
        else:
            serverinfo = None
        if serverinfo is not None:
            if serverinfo.aq_parent != self:
                req.RESPONSE.expireCookie('pref_user_format')
                return 0
            else:
                medianet = getToolByName(self, 'portal_medianet')
                isLive = medianet.isLive(self)
                server = serverinfo.getMediaServer()
                if isLive:
                    searchword = 'live'
                else:
                    searchword = 'on-demand'
                has_type = string.find(string.lower(server.meta_type), searchword)
                if has_type == -1:
                    req.RESPONSE.expireCookie('pref_user_format')
                    return 0
                else:
                    return 1
        else:
            req.RESPONSE.expireCookie('pref_user_format')
            return 0

    security.declarePublic('getFormats')
    def getFormats(self):
        """
        Return the formats that this event contains
        """
        resp = self.REQUEST.RESPONSE
        medianet = getToolByName(self, 'portal_medianet')

        if medianet.isLive(self):
            searchword = 'live'
        else:
            searchword = 'ondemand'

        serverinfos = self.objectValues('Media Manager ServerInfo')

        servers = []
        for serverinfo in serverinfos:
            server = serverinfo.getMediaServer()
            has_word = string.find(string.lower(server.type), searchword)
            if has_word != -1:
                if serverinfo.getValueFor('isEnabled'):
                    servers.append(serverinfo)

        return servers

    security.declareProtected(CMFCorePermissions.View, 'getCinemaSkin')
    def getCinemaSkin(self, RESPONSE=None):
        """
        Return the cinema skin to use in mediaevent_view
        """
        req = self.REQUEST
        medianet = getToolByName(self, 'portal_medianet')
        isLive = medianet.isLive(self)
        masterfile = self.listFileObjects()[0]
        serverinfo = None
        if isLive:
            searchword = 'live'
        else:
            searchword = 'ondemand'

        # check if the user has any preferences for the streaming format for this event
        userpref = req.cookies.get('pref_user_format', None)
        if userpref is not None:
            try:
                serverinfo = self.restrictedTraverse(userpref)
            except:
                serverinfo  = None
        else:
            serverinfo = medianet.getServerinfoFor(self)
            # check if the preferred format is for live events
            # and the event is no longer live or vice versa
        if serverinfo:
            mediaserver = serverinfo.getMediaServer()
            has_type = string.find(string.lower(mediaserver.type), searchword)
            if has_type == -1:
                req.RESPONSE.expireCookie('pref_user_format')
                if RESPONSE is not None:
                    return RESPONSE.redirect("%s/mediaevent_view" % self.absolute_url())
            else:
                skin = serverinfo.getValueFor('cinemaskin')
                return skin
        else:
            req.RESPONSE.expireCookie('pref_user_format')
            if RESPONSE is not None:
                return RESPONSE.redirect("%s/mediaevent_view" % self.absolute_url())

    security.declarePublic('setEditFormat')
    def setEditFormat(self, serverinfo_id, REQUEST, redirect=1):
        """
        Set the preferred format to use with this event when editing it
        """
        zLOG.LOG(self.meta_type, zLOG.ERROR, 'Set EDIT FORMAT cookie called')
        resp = REQUEST.RESPONSE
        resp.setCookie('pref_edit_format', serverinfo_id, path='/'+self.absolute_url(relative=1))
        if REQUEST.cookies.has_key('pref_edit_format'):
            extra = ''
        else:
            extra = '?noCookies=true'
        if redirect:
            req_url = "%s/edit_suite%s" % (self.absolute_url(), extra)
            resp.redirect(req_url)

    security.declareProtected(CMFCorePermissions.View, 'validEditFormat')
    def validEditFormat(self):
        """
        Returns true if the user has a valid preferred edit format
        """
        req = self.REQUEST
        userpref = req.cookies.get('pref_edit_format',None)
        if userpref:
            try:
                serverinfo = self.restrictedTraverse(userpref)
            except:
                serverinfo = None
        else:
            serverinfo = None
        if serverinfo is not None:
            if serverinfo.aq_parent != self:
                req.RESPONSE.expireCookie('pref_edit_format')
                return 0
            else:
                return 1
        else:
            req.RESPONSE.expireCookie('pref_edit_format')
            return 0

    security.declareProtected(CMFCorePermissions.View, 'getIcon')
    def getIcon(self):
        medianet = getToolByName(self, 'portal_medianet')
        if medianet.isLive(self):
            return 'mediaevent_icon_scheduled.gif'
        else:
            return 'mediaevent_icon_archived.gif'

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setEncoderInformation')
    def setEncoderInformation(self, serverinfoid, encoder_path, ondemand_server, profileid=None):
        """
        Set the information about the used encoder for this event
        """
        encoder_list = getattr(self, '_last_encoders', [])
        has_enc = 0
        for encdict in encoder_list:
            if encdict['serverinfo'] == serverinfoid:
                has_enc = 1
                encoder_dict = encdict
        if has_enc:
            encoder_dict['encoder'] = encoder_path
            encoder_dict['profile'] = profileid
            encoder_dict['ondemand_server'] = ondemand_server
        else:
            encoder_dict = {'serverinfo': serverinfoid
                                   , 'encoder': encoder_path
                                   , 'profile': profileid
                                   , 'ondemand_server': ondemand_server}
            encoder_list.append(encoder_dict)
        self._last_encoders = encoder_list
        self._p_changed = 1

    security.declareProtected(CMFCorePermissions.ManageProperties, 'getEncoderInformation')
    def getEncoderInformation(self, serverinfoid):
        """
        Return a dictionary containing info about encoder and profile for the requested serverinfo
        """
        for encdict in getattr(self, '_last_encoders', []):
            if encdict['serverinfo'] == serverinfoid:
                return {'encoder': encdict['encoder'], 'profile': encdict['profile'], 'ondemand_server': encdict['ondemand_server']}

    security.declareProtected(CMFCorePermissions.View, 'hasServerInfoForServer')
    def hasServerInfoForServer(self, server):
        """
        Returns a boolean value indicating if this event has a ServerInfo object
        corresponding with the server.
        The 'server' parameter can be a string containing a traversable path
        or the server object itself.
        """
        if type(server) == type(''):
            try:
                server = self.restrictedTraverse(server)
            except:
                return 0
        sis = self.objectValues('Media Manager ServerInfo')
        for si in sis:
            if si.getMediaServer() == server:
                return 1

    security.declarePublic('manage_FTPlist')
    def manage_FTPlist(self, REQUEST):
        """
        Return the ftp list
        """
        import marshal
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Manage ftp list called')
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Object items is %s' % str(self.objectItems()))
        for k,v in self.objectItems():
            zLOG.LOG(v.meta_type, zLOG.INFO, 'K is %s' % k)
            try:
                stat=marshal.loads(v.manage_FTPstat(REQUEST))
            except:
                stat=None
                zLOG.LOG(self.meta_type, zLOG.ERROR, \
                                'Exception in getting stat', error=sys.exc_info())
        ftp_list = PortalFolder.manage_FTPlist(self, REQUEST)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Ftp list is %s' % str(ftp_list))
        return ftp_list

def addMediaEvent(self, id, REQUEST=None):
        try:
            medianet = getToolByName(self, 'portal_medianet')
        except:
            raise "You must have a Media Manager Medianet named 'medianet' in the portal root!!!"
        ob=MediaEvent(id)
        self._setObject(id, ob)
        #event = self._getOb(id)
        #event._initializeEvent()

Globals.InitializeClass(MediaEvent)
