##############################################################################
#    Copyright (C) 2001-2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

#Globals
import Globals, zLOG, os, string
from AccessControl import ClassSecurityInfo
from DateTime import DateTime
from OFS.Folder import Folder
from Products.PageTemplates import PageTemplateFile
from types import TupleType, ClassType, StringType
from Interface import Base as InterfaceClass

#CMF
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName, UniqueObject
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

# MyMediaManager
from Products.MyMediaManager.General import audioformats, videoqualities, audienceratings, bandwidths
from Products.MyMediaManager.interfaces.IPluginBase import IPluginBase
from Products.MyMediaManager.interfaces.IServerPlugin import IServerPlugin
from Products.MyMediaManager.interfaces.IMetaGeneratorPlugin import IMetaGeneratorPlugin
from Products.MyMediaManager.interfaces.ILiveServerPlugin import ILiveServerPlugin
from Products.MyMediaManager.interfaces.IOnDemandServerPlugin import IOnDemandServerPlugin
from Products.MyMediaManager.interfaces.IEncoderPlugin import IEncoderPlugin
from Products.MyMediaManager.interfaces.ISlideshowEncoder import ISlideshowEncoder

from utils import getUniqueId
import MediaPermissions
_zptdir = os.path.join(Globals.package_home(globals()), 'www')

class Medianet( UniqueObject, PortalFolder, ActionProviderBase):
    """
    A medianet is used for storing metadata about media servers.
    Servers are provided as plugins.
    """

    id = 'portal_medianet'
    meta_type = 'Media Manager Medianet'
    _registered_server_plugins = []
    _registered_meta_generators = []
    _registered_encoder_plugins = []
    _installed_plugins = []
    _default_liveserver = None
    _default_ondemandserver = None
    _known_formats = {
                         'audio/vnd.rn-realaudio': 'Real Audio'
                        ,'application/vnd.rn-realmedia': 'Real Media'
                        , 'video/x-ms-asf': 'Windows Media'
                     }

    _actions = [ ActionInformation(id='manage_medianet'
                    , title='Setup medianet'
                    , action=Expression(text='string: ${portal_url}/portal_medianet/medianet_admin_form')
                    , condition=Expression(text="python:'portal_configuration' in portal.objectIds()")
                    , permissions=(MediaPermissions.ManageMedianet,)
                    , category='configuration'
                    , visible=1
                    ),
                ActionInformation(id='manage_medianet'
                    , title='Setup medianet'
                    , action=Expression(text='string: ${portal_url}/portal_medianet/medianet_admin_form')
                    , condition=Expression(text="python:'portal_configuration' not in portal.objectIds()")
                    , permissions=(MediaPermissions.ManageMedianet,)
                    , category='global'
                    , visible=1
                    )]

    manage_options=({'label':'Contents', 'action':'manage_main',}
                   , {'label':'Server Plugins', 'action':'manage_server_plugins'}
                   , {'label':'Encoder Plugins', 'action':'manage_encoder_plugins'}
                   , {'label':'Meta Generators', 'action':'manage_metagenerator_plugins'}
                   , {'label':'Streaming Formats', 'action':'manage_formats'}
                   ) + Folder.manage_options[1:]

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainMedianet', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_server_plugins')
    manage_server_plugins = PageTemplateFile.PageTemplateFile('manageMedianetServerPlugins', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_encoder_plugins')
    manage_encoder_plugins = PageTemplateFile.PageTemplateFile('manageMedianetEncoderPlugins', _zptdir)
    
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_metagenerator_plugins')
    manage_metagenerator_plugins = PageTemplateFile.PageTemplateFile('manageMetageneratorPlugins', _zptdir)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_formats')
    manage_formats = PageTemplateFile.PageTemplateFile('manageMedianetFormats', _zptdir)

    audioformats = audioformats
    videoqualities = videoqualities
    audienceratings = audienceratings
    bandwidths = bandwidths


    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return the actions for the tool
        """
        return self._actions

    def __init__(self):
        PortalFolder.__init__(self, self.id)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Edit the data of the medianet
        """
        req = REQUEST
        self.setTitle(req.get('title', None))
        self.setDescription(req.get('description', None))
        self.reindexObject()
        msg = 'Medianet metadata updated'
        req.RESPONSE.redirect('%s/medianet_admin_form?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

    #
    # Utility methods
    #
    
    security.declarePublic('getUniqueID')
    def getUniqueID(self):
        """
        Return a unique identifier
        """
        return getUniqueId()

    #
    # Server plugin methods
    #
    
    security.declarePublic('getDefaultServer')
    def getDefaultServer(self, type=None):
        """
        Return the default live- or ondemand server according to type attribute
        """
        if type == 'live':
            server = self._default_liveserver
            return self._default_liveserver
        elif type == 'ondemand':
            return self._default_ondemandserver
        else:
            raise 'Attribute Error', "Type is wrong. It's %s and it should be either 'live' or 'ondemand'" % type

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setDefaultServer')
    def setDefaultServer(self, REQUEST):
        """
        Set the default server (live/ondemand) for this medianet
        """
        req = REQUEST
        type = req.get('type', None)
        serverid = req.get('serverid', None)
        if type:
            if type == 'live':
                self._default_liveserver = getattr(self, serverid)
                msg = 'Default live server set'
            elif type == 'ondemand':
                self._default_ondemandserver = getattr(self, serverid)
                msg = 'Default on-demand server set'
            else:
                raise 'Attribute Error', "Type is wrong. It's %s and it should be either 'live' or'ondemand'" % type
        else:
            raise 'Attribute Error', 'Type was not supplied'

        req.RESPONSE.redirect('%s/medianet_admin_form?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

    security.declareProtected(CMFCorePermissions.ManagePortal, 'addServer')
    def addServer(self, REQUEST):
        """
        Add a new mediaserver to the medianet
        """
        req = REQUEST
        typestool = getToolByName(self, 'portal_types')
        typename = req.get('servertype', None)
        addtype = None
        for cti in typestool.listTypeInfo():
            if cti.Metatype() == typename:
                addtype = cti.Title()
        if addtype is None:
            raise "ValueError", "Entry for meta type %s not found in portal_types"
        id = getUniqueId()
        self.invokeFactory(id=id, type_name=addtype)
        server = getattr(medianet, id)
        server.setTitle(req.get('title', None))
        msg = 'Added new server of type: %s' % typename
        req.RESPONSE.redirect('%s/medianet_admin_form?%sportal_status_message=%s' % (medianet.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

    security.declareProtected(CMFCorePermissions.ManagePortal, 'deleteServer')
    def deleteServer(self, REQUEST):
        """
        Delete a media server from the medianet.
        """
        req = REQUEST
        serverid = req.get('serverid', None)
        server = getattr(self, serverid, None)
        if serverid and server:
            def_live = self._default_liveserver
            def_archive = self._default_ondemandserver
            if def_live == server:
                self._default_liveserver = None
            if def_archive == server:
                self._default_ondemandserver = None
            servertitle = server.title_or_id()
            self._delObject(serverid)
            msg = 'Server %s has been deleted' % servertitle
            req.RESPONSE.redirect('%s/medianet_admin_form?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))
        else:
            raise 'Attribute Error', 'Serverid was not supplied'

    security.declarePublic('getServerById')
    def getServerById(self, id):
        """
        Return the server object matching the id
        """
        if hasattr(self, id):
            # XXX Check if this is really a server object
            return getattr(self, id)
        else:
            raise 'Attribute Error', "Server with id '%s' not found" % id

    security.declarePublic('listServers')
    def listServers(self, servertype=None):
        """
        Returns a list containing server objects in the medianet.
        """
        if servertype:
            types = []
            for plugin in self.listServerPlugins():
                metatype = plugin['meta_type']
                has_type = string.find(string.lower(metatype), servertype)
                if not has_type == -1:
                    types.append(metatype)
            if types:
                return self.objectValues(types)
            else:
                return None
        else:
            objs = self.contentValues()
            servers = []
            for obj in objs:
                if IServerPlugin.isImplementedBy(obj):
                    servers.append(obj)
            return servers

    security.declarePublic('listServerPlugins')
    def listServerPlugins(self, only_installed=0):
        """
        Returns a list of dictionaries containing registered
        servertypes meta_type/provides/installed.
        """
        plugins = []
        for plugin in self._registered_server_plugins:
            if not only_installed or self.isPluginInstalled(plugin):
                plugins.append({
                            'meta_type':plugin.meta_type
                            , 'provides':plugin._provides
                            , 'installed':self.isPluginInstalled(plugin)
                            })
        return plugins

    security.declarePublic('serverIsLive')
    def serverIsLive(self, server):
        """
        Return a boolean indication whether the server is live or not
        """
        isLive = 0
        if ILiveServerPlugin.isImplementedBy(server):
            isLive = 1
        return isLive

    security.declarePublic('serverIsOndemand')
    def serverIsOndemand(self, server):
        """
        Return a boolean indication whether the server is live or not
        """
        isOndemand = 0
        if IOnDemandServerPlugin.isImplementedBy(server):
            isOndemand = 1
        return isOndemand

    security.declarePublic('pluginImplements')
    def implementsSlideshow(self, obj):
        """
        Test whether an object implements the slideshow interface
        """
        return ISlideshowEncoder.isImplementedBy(obj)

    security.declarePublic('getProfileDataForServerInfo')
    def getEncoderProfileForServerInfo(self, serverinfo):
        """
        Return a dictionary of encoder profile values.
        See interfaces/IEncoderPlugin for details
        """
        profile = serverinfo.getValueFor('encoder_profile')
        if profile:
            try:
                obj = self.restrictedTraverse(profile)
                return obj
            except:
                zLOG.LOG(self.meta_type, zLOG.INFO, 'Could not get profile object %s' % profile)
        return None
        
    security.declarePublic('listEncoderProfilesForServer')
    def listEncoderProfilesForServer(self, server):
        """
        Returns a list of encoder ojects that can be used with the specific server
        """
        tools = self.getEncoderTools()
        search_tools = []
        for provide in server._provides:
            for tool in tools:
                if provide in tool._provides:
                    search_tools.append(tool.id)
        catalog = getToolByName(self, 'portal_catalog')
        brains = catalog(meta_type='Media Manager Encoder Profile', encoder=search_tools)
        profiles = []
        for brain in brains:
            ob = brain.getObject()
            if not ob in profiles:
                profiles.append(ob)
        return profiles

    security.declarePublic('hasLivePlugin')
    def hasLivePlugin(self):
        """
        Test whether there are any live plugins installed
        """
        hasLive = 0
        for plugin_metatype in self._installed_plugins:
            plugin = self.getPluginByMetatype(plugin_metatype)
            if ILiveServerPlugin.isImplementedByInstancesOf(plugin):
                hasLive = 1
        return hasLive

    #
    # Encoder plugin methods
    #

    security.declarePublic('listServersForEncoder')
    def listServersForEncoder(self, encoder_id):
        """
        Returns a list containing server objects that can be used be a specific encoder
        """
        enctool = getToolByName(self, encoder_id)
        servers = []
        for server in self.listServers('live'):
            for provide in server._provides:
                if provide in enctool._provides and not server in servers:
                    servers.append(server)
        return servers

    security.declarePublic('listEncoderPlugins')
    def listEncoderPlugins(self, only_installed=0):
        """
        Returns a list of dictionaries containing registered
        servertypes meta_type/provides/installed.
        """
        plugins = []
        for plugin in self._registered_encoder_plugins:
            if not only_installed or self.isPluginInstalled(plugin):
                plugins.append({
                            'meta_type':plugin.meta_type
                            , 'provides':plugin._provides
                            , 'installed':self.isPluginInstalled(plugin)
                            })
        return plugins

    security.declarePublic('getEncoderTools')
    def getEncoderTools(self):
        """
        Return a list of all available encoder tools
        """
        plugs = self.listAvailableEncoderTools()
        tools = []
        for plug in plugs:
            tool = getToolByName(self, plug)
            tools.append(tool)
        return tools

    security.declareProtected(MediaPermissions.StreamLive, 'listAvailableEncoderTools')
    def listAvailableEncoderTools(self):
        """
        Returns a list containing ids of installed encoder plugins
        """
        plugins = []
        for plugin in self._registered_encoder_plugins:
            if self.isPluginInstalled(plugin):
                plugins.append(plugin.id)
        return plugins

    security.declarePublic('listMetageneratorPlugins')
    def listMetageneratorPlugins(self, only_installed=0):
        """
        Returns a list of dictionaries containing registered
        metagenerator meta_type/provides/installed.
        """
        plugins = []
        for plugin in self._registered_meta_generators:
            if not only_installed or self.isPluginInstalled(plugin):
                plugins.append({
                            'meta_type':plugin.meta_type
                            , 'installed':self.isPluginInstalled(plugin)
                            })
        return plugins

    security.declarePublic('listMetaGenerators')
    def listMetaGenerators(self):
        """
        Returns a list of containing meta types of installed metagenerators
        """
        plugins = []
        for plugin in self._registered_meta_generators:
            if self.isPluginInstalled(plugin):
                plugins.append(plugin.meta_type)
        return plugins

    #
    # General plugin methods
    #

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_installPlugin')
    def manage_installPlugin(self, meta_type, ptype, REQUEST):
        """
        Performes installation for a registered plugin
        """
        retval = 0
        typestool = getToolByName(self, 'portal_types')
        skinstool = getToolByName(self, 'portal_skins')
        portalroot = getToolByName(self, 'portal_url').getPortalObject()
        installed_plugins = self._installed_plugins
        for plugin in self._registered_server_plugins:
            #Installing server plugins
            if plugin.meta_type == meta_type and not self.isPluginInstalled(plugin):
                retval = plugin.install(plugin, typestool, skinstool)
                installed_plugin = plugin.meta_type
        for plugin in self._registered_encoder_plugins:
            #Installing encoder plugins
            if plugin.meta_type == meta_type and not self.isPluginInstalled(plugin):
                retval = plugin.install(plugin, typestool, skinstool, portalroot)
                installed_plugin = plugin.meta_type
        for plugin in self._registered_meta_generators:
            #Installing metagenerator plugins
            if plugin.meta_type == meta_type and not self.isPluginInstalled(plugin):
                retval = plugin.install(plugin, typestool, skinstool)
                installed_plugin = plugin.meta_type
        if retval:
            installed_plugins.append(installed_plugin)
            msg = "Installation succeeded"
        else:
            msg = "Installation failed or plugin couldn't be found"
        self._installed_plugins = installed_plugins
        self._p_changed = 1

        if ptype == 'Encoder':
            manage_method = self.manage_encoder_plugins
        elif ptype == 'Server':
            manage_method = self.manage_server_plugins
        elif ptype == 'Metagenerator':
            manage_method = self.manage_metagenerator_plugins
        else:
            raise "Bad Request", "parameter ptype should be Encoder or Server"

        if not REQUEST.has_key('no_redirect'):
            return manage_method(
                self, REQUEST, manage_tabs_message=msg)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_uninstallPlugin')
    def manage_uninstallPlugin(self, meta_type, ptype, REQUEST):
        """
        Performes uninstallation for a registered plugin
        """
        retval = 0
        typestool = getToolByName(self, 'portal_types')
        skinstool = getToolByName(self, 'portal_skins')
        portalroot = getToolByName(self, 'portal_url').getPortalObject()
        installed_plugins = self._installed_plugins
        for plugin in self._registered_server_plugins:
            #Un-installing server plugins
            if plugin.meta_type == meta_type and self.isPluginInstalled(plugin):
                retval = plugin.uninstall(plugin, typestool, skinstool)
                uninstalled_plugin = plugin.meta_type
        for plugin in self._registered_encoder_plugins:
            if plugin.meta_type == meta_type and self.isPluginInstalled(plugin):
                retval = plugin.uninstall(plugin, typestool, skinstool, portalroot)
                uninstalled_plugin = plugin.meta_type
        for plugin in self._registered_meta_generators:
            if plugin.meta_type == meta_type and self.isPluginInstalled(plugin):
                retval = plugin.uninstall(plugin, typestool, skinstool)
                uninstalled_plugin = plugin.meta_type
        if retval:
            msg = "De-installation succeeded"
            i = 0
            for plugin in self._installed_plugins:
                if plugin == uninstalled_plugin:
                    delindex = i
                i = i +1
            del installed_plugins[delindex]
        else:
            msg = "De-installation failed or plugin couldn't be found"
        self._installed_plugins = installed_plugins
        self._p_changed = 1

        if ptype == 'Encoder':
            manage_method = self.manage_encoder_plugins
        elif ptype == 'Server':
            manage_method = self.manage_server_plugins
        elif ptype == 'Metagenerator':
            manage_method = self.manage_metagenerator_plugins
        else:
            raise "Bad Request", "parameter ptype should be Encoder or Server"

        if not REQUEST.has_key('no_redirect'):
            return manage_method(
                self, REQUEST, manage_tabs_message=msg)

    security.declarePublic('listRegisteredPlugins')
    def listRegisteredPlugins(self, only_installed=0):
        """
        Returns a list of dictionaries containing registered
        plugins meta_type/provides/installed.
        """
        plugins = []
        registered_plugins = self._registered_server_plugins + self._registered_encoder_plugins
        for plugin in registered_plugins:
            if not only_installed or self.isPluginInstalled(plugin):
                meta = plugin.meta_type
                provides = getattr(plugin, '_provides', '')
                installed = self.isPluginInstalled(plugin)
                plugins.append({
                            'meta_type': meta
                            , 'provides':provides
                            , 'installed': installed
                            })
        return plugins

    security.declarePublic('getPluginByMetatype')
    def getPluginByMetatype(self, meta_type):
        """
        Return the plugin corresponding the meta_type
        """
        plu = None
        for plugin in self._registered_server_plugins:
            if plugin.meta_type == meta_type:
                plu = plugin
        return plu

    security.declarePublic('isPluginInstalled')
    def isPluginInstalled(self, plugin):
        """
        Tests whether the plugin is installed or not
        """
        typestool = getToolByName(self, 'portal_types')
        skinstool = getToolByName(self, 'portal_skins')
        portalroot = getToolByName(self, 'portal_url').getPortalObject()
        try:
            fti = plugin.factory_type_information[0]
            specialtype = None
        except:
            fti = None
            try:
                specialtype = plugin.special_type
            except:
                specialtype = None
        skinname = plugin.skinname

        in_list = plugin.meta_type in self._installed_plugins
        if fti:
            in_types = fti['id'] in typestool.objectIds()
        if specialtype and specialtype == 'tool':
            in_types = plugin.id in portalroot.objectIds()
        has_skinfolder = skinname in skinstool.objectIds()

        skins = skinstool.getSkinSelections()
        is_skin_listed = 1
        for skin in skins:
            path = skinstool.getSkinPath(skin)
            path = map(string.strip, string.split(path,','))
            if skinname not in path:
                is_skin_listed = 0

        is_installed = (in_list and in_types and has_skinfolder and is_skin_listed)
        return is_installed

    #
    # Format (Content-type) methods
    #

    security.declarePublic('getFormatNameByType')
    def getFormatNameByType(self, content_type):
        """
        Return a human readable name based on a content type.
        If the content_type is not known we return either the
        last part of the content type or the content type itself.
        """
        if content_type in self._known_formats.keys():
            return self._known_formats[content_type]
        else:
            l = content_type.split('/')
            if len(l) > 1:
                return l[1]
            else:
                return content_type

    security.declarePublic('listKnownFormats')
    def listKnownFormats(self):
        """
        Return a dictinary of mimetype/human-readable format
        """
        return self._known_formats

    security.declarePublic('listRegisteredFormats')
    def listRegisteredFormats(self):
        """
        Return a tuple of mimetypes that the registered and installed plugins provide
        """
        types = []
        for plugin in self.listServerPlugins(only_installed=1):
           for p in plugin['provides']:
               if not p in types:
                   types.append(p)
        return types

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addStreamingFormat')
    def manage_addStreamingFormat(self, REQUEST):
        """
        Performes uninstallation for a registered plugin
        """
        major = REQUEST.get('content_type_major','')
        minor = REQUEST.get('content_type_minor','')
        msg = ""

        if not major or not minor:
            msg = "You have to fill in both the major and the minor part of content type"
        else:
            format = '%s/%s' % (major, minor)

            formats = self._known_formats
            if not formats.has_key(format):
                formats[format] = REQUEST.get('content_type_name','')

                self._known_formats = formats
                self._p_changed = 1
                msg = "Format added"
            else:
                msg = "%s already registered" % format
        
        if not REQUEST.has_key('no_redirect'):
            return self.manage_formats(
                self, REQUEST, manage_tabs_message=msg)

    #
    # Meta generator methods
    #
    
    security.declarePublic('listMetaGeneratorPlugins')
    def listMetaGeneratorPlugins(self):
        """
        Returns a list of registered meta generators
        """
        plugins = []
        for plugin in self._registered_meta_generators:
            plugins.append(plugin)
        return plugins

    #
    # MediaEvent methods
    #
        
    security.declareProtected(CMFCorePermissions.View, 'playEvent')
    def playEvent(self, event, useMetaGenerator=1):
        """
        Returns a URL that is the link for playing the event
        """

    security.declarePublic( 'getEventFor' )
    def getEventFor(self, content, path = None):
        """
            Return the event object for content, can be itself.
            The returned event is not necessarily the event in
            which the object is contained but the event in
            which the object is shown.
            The path is the url of the topmost frame ('/Members/foobar/MyClip') - if it is 'None'
            the returned value is either the event in which the
            object is contained or 'None'.
        """
        if content.meta_type == 'Media Manager Event':
            return content
        event = None
        tmp = 1
        if path is None:
            tmp = content
            while event is None and tmp is not None and not getattr(tmp, '_isPortalRoot', 0):
                if hasattr(tmp, 'aq_parent'):
                    tmp = tmp.aq_inner.aq_parent #getattr(tmp, 'aq_parent')
                else:
                    tmp = tmp.restrictedTraverse('..') #aq_parent(aq_inner(tmp))
                if tmp.meta_type == "Media Manager Event":
                    event = tmp
                    break
        else:
            tmp = self.restrictedTraverse(path, None)
            while event is None and tmp is not None and not getattr(tmp, '_isPortalRoot', 0):
                tmp = aq_parent(aq_inner(ob))
                if tmp.meta_type == "Media Manager Event":
                    event = tmp
        if (event is not None and event.meta_type == "Media Manager Event"):
            return event
        else:
            return None

    security.declarePublic( 'getSlideshowObjectFor' )
    def getSlideshowObjectFor(self, content):
        """
            Return the slide show object for content).
        """
        if content.meta_type == "Media Manager Slide Show":
            return content
        event = self.getEventFor(content)
        if event is None:
            return None #raise "Couldn't find event for %s" % content.getId()

        slides = event.objectValues('Media Manager Slide Show')
        if slides:
            return slides[0]
        return None

    security.declarePublic('getMetagenobjFor')
    def getMetagenobjFor(self, content, context='cinema'):
        """
        Return a metagenerator for the content or None
        """
        event = self.getEventFor(content)
        req = self.REQUEST
        metagen = None
        if context == 'cinema':
            cookiename = 'pref_user_format'
        else:
            cookiename = 'pref_edit_format'
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Cookie name is %s' % cookiename)
        zLOG.LOG(self.meta_type, zLOG.INFO, 'Cookies are %s' % str(req.cookies))
        if req.cookies.has_key(cookiename):
            serverinfo_path = req.cookies.get(cookiename)
            serverinfo = self.unrestrictedTraverse(serverinfo_path)
            metagenpath = serverinfo.getValueFor('meta_generator', info_only=1)
            if metagenpath:
                metagen = self.restrictedTraverse(metagenpath)

        if not metagen:
            masterfile = event.listFileObjects()[0]
            serverinfos = masterfile.listServerInfos()
            if self.isLive(event):
                sword = 'live'
            else:
                sword = 'ondemand'
            for serverinfo in serverinfos:
                mserver = serverinfo.getMediaServer()
                hasword = string.find(string.lower(mserver.type), sword)
                if hasword != -1:
                    metagenpath = serverinfo.getValueFor('meta_generator', info_only=1)
                    if metagenpath:
                        metagen = self.restrictedTraverse(metagenpath)
                    break

        return metagen

    security.declarePublic('getServerinfoFor')
    def getServerinfoFor(self, content):
        """
        Return the serverinfo to be used for playing content.
        If request has not serverinfo, we will try and find on in an event
        """
        #zLOG.LOG(self.meta_type, zLOG.INFO, "getServerinfoFor %s" % content.absolute_url())
        info = None
        req = self.REQUEST
        if req.cookies.has_key('pref_user_format'):
            serverinfopath = req.cookies.get('pref_user_format')
            serverinfo = self.portal_url.getPortalObject().unrestrictedTraverse(serverinfopath)
            info = serverinfo
        else:
            event = self.getEventFor(content)
            masterfile = event.listFileObjects()[0]
            serverinfos = masterfile.listServerInfos()
            if self.isLive(event):
                sword = 'live'
            else:
                sword = 'ondemand'
            for serverinfo in serverinfos:
                mserver = serverinfo.getMediaServer()
                hasword = string.find(string.lower(mserver.type), sword)
                if hasword != -1:
                    info = serverinfo
                    break
        return info

    security.declarePublic('getEditServerInfo')
    def getEditServerInfo(self, content):
        """
        Return the serverinfo to be used for playing content.
        If request has not serverinfo, we will try and find one in an event
        """
        info = None
        req = self.REQUEST
        if req.cookies.has_key('pref_edit_format'):
            serverinfopath = req.cookies.get('pref_edit_format')
            serverinfo = self.restrictedTraverse(serverinfopath)
            info = serverinfo
        else:
            event = self.getEventFor(content)
            masterfile = event.listFileObjects()[0]
            serverinfos = masterfile.listServerInfos()
            for serverinfo in serverinfos:
                mserver = serverinfo.getMediaServer()
                hasword = string.find(string.lower(mserver.type), 'ondemand')
                if hasword != -1:
                    info = serverinfo
                    break
        return info

    security.declarePublic( 'isLive' )
    def isLive(self, content):
        """
        Find out whether the event is being broadcasted live or is archived for ondemand broadcast
        """
        status = self.status(content)
        return status in ('prelive','live')

    security.declarePublic( 'status' )
    def status(self, content):
        """
        Find out whether the event is being broadcasted live or is archived for ondemand broadcast
        """
        e = self.getEventFor(content)
        if not e:
            raise MediaManagerException, "Couldn't find event for %s" % content.absolute_url(relative=1)

        _15min = 900000
        now = DateTime().millis()
        start = e.start_date.millis()
        end = e.end_date.millis()  # 15 min. after

        if now > start and now < end:
            return 'live'
        elif now < start:
            return 'prelive'
            """
            elif now > end and now < (end + _15min):
                return 'passed'
            """
        elif now > end: # XXX archived is actually first when the masterfile has been uploaded.
            return 'archived'     # how do I know that.
        else:
            return 'undefined'

    security.declarePublic('getEditorPlayerFor')
    def getEditorPlayerFor(self, content, REQUEST):
        """
        Return the template macro to be used for editing 'content'
        """
        event = self.getEventFor(content)
        serverid = REQUEST.get('serverid', None)
        serverinfo = None
        if serverid:
            masterfile = event.listFileObjects()[0]
            serverinfo = masterfile.getServerInfo(serverid=serverid)
        else:
            userformat = REQUEST.cookies.get('pref_edit_format', None)
            if not userformat:
                raise 'AttributeError', 'No userformat selected'
        if not serverinfo:
            serverinfo = self.restrictedTraverse(userformat)
        mediaserver = serverinfo.getMediaServer()
        templatename = mediaserver.getEditorPlayerSkin()
        template = getattr(self, templatename)
        return template.macros['master']

    security.declarePublic('getCinemaPlayerFor')
    def getCinemaPlayerFor(self, content, REQUEST):
        """
        Return the template macro to be used for playing 'content'
        """
        event = self.getEventFor(content)
        serverid = REQUEST.get('serverid', None)
        if serverid:
            masterfile = event.listFileObjects()[0]
            serverinfo = masterfile.getServerInfo(serverid=serverid)
        else:
            userformat = REQUEST.cookies.get('pref_user_format', None)
            if not userformat:
                serverinfo = self.getServerinfoFor(content)
            else:
                serverinfo = self.restrictedTraverse(userformat)
        if not serverinfo:
            raise 'AttributeError', 'No appropiate server info found'
        mediaserver = serverinfo.getMediaServer()
        templatename = mediaserver.getCinemaPlayerSkin()
        template = getattr(self, templatename)
        return template.macros['master']

    security.declarePublic('getEditorUrlFor')
    def getEditorUrlFor(self, content, REQUEST):
        """
        Get the editor url to be used in the editor player
        """
        event = self.getEventFor(content)
        serverid = REQUEST.get('serverid', None)
        serverinfo = None
        if serverid:
            masterfile = event.listFileObjects()[0]
            serverinfo = masterfile.getServerInfo(serverid=serverid)
        else:
            userformat = REQUEST.cookies.get('pref_edit_format', None)
            if not userformat:
                raise 'AttributeError', 'No userformat selected'
        if not serverinfo:
            serverinfo = self.restrictedTraverse(userformat)
        mediaserver = serverinfo.getMediaServer()
        masterfile = serverinfo.getMasterfile()
        url = mediaserver.getPlayUrl(masterfile.getFileName())
        return url

    security.declarePublic('getCinemaUrlFor')
    def getCinemaUrlFor(self, content, REQUEST, liveserverinfo=None):
        """
        Get the cinema player url for content.
        """
        event = self.getEventFor(content)
        serverinfo = self.getServerinfoFor(content)
        masterfile = event.listFileObjects()[0]
        filename = masterfile.getFileName()
        mediaserver = serverinfo.getMediaServer()
        if liveserverinfo is not None and liveserverinfo.meta_type == 'Media Manager ServerInfo':
            playurl = mediaserver.getPlayUrl(liveserverinfo)
        else:
            playurl = mediaserver.getPlayUrl(filename)
        return playurl

    security.declarePublic('findPluginMetatypesByFormat')
    def findPluginMetatypesByFormat(self, REQUEST, format=None):
        """
        Return a list of valid servers by the given format
        """
        if format is None:
            format = REQUEST.cookies.get('streaming_format', None)

        delivertype = REQUEST.cookies.get('deliver_type', None)
        if not format:
            raise 'AttributeError', 'No streaming format selected'

        servers = []
        for plugin_metatype in self._installed_plugins:
            plugin = self.getPluginByMetatype(plugin_metatype)
            if delivertype == 'live':
                if not ILiveServerPlugin.isImplementedByInstancesOf(plugin):
                    continue
            else:
                if not IOnDemandServerPlugin.isImplementedByInstancesOf(plugin):
                    continue
            for reg_plugin in self.listServerPlugins():
                if reg_plugin['meta_type'] == plugin_metatype:
                    if format in reg_plugin['provides']:
                        servers.append(reg_plugin['meta_type'])

        return servers

    #
    # Server add wizard
    #
    
    security.declareProtected(MediaPermissions.ManageMedianet, 'addServer_step1')
    def addServer_step1(self, REQUEST, RESPONSE):
        """
        Add a media server - step 1
        """
        delivertype = REQUEST.get('deliver_type', None)
        if not delivertype:
            redir = 'back'
            msg = 'Content Delivery Type not selected'
        else:
            RESPONSE.setCookie('deliver_type', delivertype, path='/')
            redir = 'step2'
            msg = 'Step 1 completed'

        if redir == 'back':
            RESPONSE.redirect('%s/start_server_wizard?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))
        else:
            RESPONSE.redirect('%s/serverwizard_step2?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

    security.declareProtected(MediaPermissions.ManageMedianet, 'addServer_step2')
    def addServer_step2(self, REQUEST, RESPONSE):
        """
        Add a media server - step 2
        """
        format = REQUEST.get('stream_format', None)
        if not format:
            redir = 'back'
            msg = 'Streaming Format not selected'
        else:
            RESPONSE.setCookie('streaming_format', format, path='/')
            redir = 'step3'
            msg = 'Step 2 completed'

        if redir == 'back':
            RESPONSE.redirect('%s/serverwizard_step2?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))
        else:
            RESPONSE.redirect('%s/serverwizard_step3?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

    security.declareProtected(MediaPermissions.ManageMedianet, 'addServer_step3')
    def addServer_step3(self, REQUEST, RESPONSE):
        """
        Add a media server - step 3
        """
        server_type = REQUEST.get('stream_server', None)
        if not server_type:
            redir = 'back'
            msg = 'Streaming Server not selected'
        else:
            RESPONSE.setCookie('streaming_server', server_type, path='/')
            redir = 'step4'
            msg = 'Step 3 completed'

        if redir == 'back':
            RESPONSE.redirect('%s/serverwizard_step3?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))
        else:
            RESPONSE.redirect('%s/serverwizard_step4?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

    security.declareProtected(MediaPermissions.ManageMedianet, 'addServer_final')
    def addServer_final(self, REQUEST, RESPONSE):
        """
        Add a media server - step 3
        """
        title = REQUEST.get('title', None)
        description = REQUEST.get('description', None)
        typename = REQUEST.cookies.get('streaming_server')
        format = REQUEST.cookies.get('streaming_format')

        if not title and not description:
            redir = 'back'
            msg = 'Title and description not supplied not selected'
        else:
            typestool = getToolByName(self, 'portal_types')
            addtype = None
            for cti in typestool.listTypeInfo():
                if cti.Metatype() == typename:
                    addtype = cti.Title()
            if not addtype:
                redir = 'back'
                msg = 'No streaming server selected'
            id = getUniqueId()
            self.invokeFactory(id=id, type_name=addtype)
            server = getattr(self, id)
            REQUEST.set('title', title)
            REQUEST.set('description', description)
            REQUEST.set('selected_format', format)
            server.edit(REQUEST)
            REQUEST.RESPONSE.expireCookie('streaming_format')
            REQUEST.RESPONSE.expireCookie('streaming_server')
            REQUEST.RESPONSE.expireCookie('deliver_type')
            redir = 'final'
            msg = 'Added new server of type: %s' % addtype
        if redir == 'back':
            RESPONSE.redirect('%s/serverwizard_step4?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))
        else:
            RESPONSE.redirect('%s/medianet_admin_form?%sportal_status_message=%s' % (self.absolute_url(), configQuery(self), string.replace(msg, ' ', '+')))

def configQuery(self):
    """
    Simple hack to check if the respons page should be framed
    """          
    hasconfig = 'portal_configuration' in self.portal_url.getPortalObject().objectIds()
    return hasconfig and 'frameable=true&' or ''

            
def classImplementsInterface(klass, iface):
    """
    return true if klass impelements iface
    """
    if iface.isImplementedByInstancesOf(klass):
        for name in iface.names():
            if not hasattr(klass, name):
                return 0
        return 1

def registerPlugin(plugin):
    """
    Registers a new plugin in the medianet
    """
    #if IServerPlugin.isImplementedByInstancesOf(plugin):
    if classImplementsInterface(plugin, IPluginBase):
        if classImplementsInterface(plugin, IServerPlugin):
            if not plugin in Medianet._registered_server_plugins:
                Medianet._registered_server_plugins.append(plugin)
        if classImplementsInterface(plugin, IMetaGeneratorPlugin):
            if not type in Medianet._registered_meta_generators:
                Medianet._registered_meta_generators.append(plugin)
        if classImplementsInterface(plugin, IEncoderPlugin):
            if not plugin in Medianet._registered_encoder_plugins:
                Medianet._registered_encoder_plugins.append(plugin)

Globals.InitializeClass(Medianet)
