##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import zLOG, string, os, Globals
from utils import getUniqueId
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner, aq_acquire
from DateTime.DateTime import DateTime
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.PageTemplates import PageTemplateFile

_zptdir = os.path.join(Globals.package_home(globals()), 'www')

class ServerInfo(SimpleItem):
    """
    A class that binds a server object to a masterfile
    """

    meta_type = 'Media Manager ServerInfo'
    security = ClassSecurityInfo()

    manage_options=({'label':'Server Info', 'action':'manage_overview',},
                   ) + SimpleItem.manage_options

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile.PageTemplateFile( 'displayServerInfo', _zptdir)

    def __init__(self, masterfile, serverid):
        """
        Make a new instance of a ServerInfo object
        """
        self.id = getUniqueId()
        self._masterfile = masterfile
        self._content_type = None
        self._serverid = serverid
        self._encoder_profile = None
        self._cinemaskin = None
        self._syndicationskin = None
        self._encoder_start = None
        self._width = None
        self._height = None
        self._isEnabled = 0

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, REQUEST=None, datadict=None):
        """
        Edit the ServerInfo object
        """
        if REQUEST:
            req = REQUEST.form
            req_url = REQUEST.get('HTTP_REFERER', None)
            has_query = string.find(req_url, '?')
            if has_query != -1:
                req_url, query = string.split(req_url, '?')
        elif datadict:
            req = datadict
            req_url = None
        else:
            raise 'AttributeError', 'Missing data that should be used to update the serverinformation'

        for key in req.keys():
            if key == 'serverid':
                self._serverid = req[key]
            elif key == 'encoder_profile':
                self._encoder_profile = req[key]
            elif key == 'cinemaskin':
                self._cinemaskin = req[key]
            elif key == 'syndicationskin':
                self._syndicationskin = req[key]
            elif key == 'height':
                self._height = req[key]
            elif key == 'width':
                self._width = req[key]
            elif key == 'content_type':
                self._content_type == req[key]
            elif key == 'isEnabled':
                self._isEnabled = int(req[key])
            elif key == 'encoder_start':
                if req[key] is not None:
                    mnet = getToolByName(self, 'portal_medianet')
                    event = mnet.getEventFor(self)
                    eventdate = event.start().strftime('%Y/%m/%d')
                    if req[key] == '':
                        encstart = event.start()
                    else:
                        encstart = "%s %s" % (str(eventdate), req[key])
                        encstart = DateTime(encstart)
                    self.setEncoderStart(encstart)
            elif key == 'edit':
                continue
            elif key == 'mediafile':
                filedata = req[key]
                if filedata:
                    self.uploadMasterfileData(filedata)
            elif key == 'filename':
                masterfile = self.getMasterfile()
                masterfile.edit(filename=req[key])
            else:
                setattr(self, '_%s' % key, req[key])

        msg = 'Server Information updated'
        if req_url:
            REQUEST.RESPONSE.redirect('%s?serverid=%s&portal_status_message=%s' % (req_url, self.getMediaServer().getId(), string.replace(msg, ' ', '+')))

    security.declarePublic('getEncoderProfile')
    def getEncoderProfile(self):
        """
        The encoder profile object or None if not found.
        """
        profile = None
        portal = self.portal_url.getPortalObject()
        path = self.getValueFor('encoder_profile')
        if path:
            try:
                profile = portal.restrictedTraverse(path)
            except:
                pass
        if not profile:
            #zLOG.LOG(self.meta_type, zLOG.INFO, "getEncoderProfile - getting value from media server")
            server = self.getMediaServer()
            value = getattr(server, '_encoder_profile', None)
            if value and callable(value):
                value = value()
            #zLOG.LOG(self.meta_type, zLOG.INFO, "getEncoderProfile - value: %s" % value)
            if value:
                try:
                    profile = portal.restrictedTraverse(value)
                except:
                    pass
        #zLOG.LOG(self.meta_type, zLOG.INFO, "getEncoderProfile - returning: %s" % profile)
        return profile

    security.declarePublic('getValueFor')
    def getValueFor(self, attr, info_only=0):
        """
        Return the value for a specific attribute
        """
        #zLOG.LOG(self.meta_type, zLOG.INFO, "getValueFor %s - dict: \n%s" % (attr, str(self.__dict__)))
        value = getattr(self, '_%s' % attr, None)
        if value and callable(value):
            value = value()
        if not value and not info_only:
            #zLOG.LOG(self.meta_type, zLOG.INFO, "getValueFor - getting value from media server")
            server = self.getMediaServer()
            value = getattr(server, '_%s' % attr, None)
            if value and callable(value):
                value = value()
        if value == 'None':
            value = None
        #zLOG.LOG(self.meta_type, zLOG.INFO, "getValueFor: %s == %s" % (attr, value))
        return value

    security.declarePublic('getContentType')
    def getContentType(self):
        """
        Return the content type of the content
        """
        if hasattr(self, '_content_type'):
            return self._content_type
        else:
            return None

    security.declarePublic('getMasterfile')
    def getMasterfile(self):
        """
        Return the masterfile object for this serverinfo object
        """
        return self._masterfile

    security.declarePublic('uploadMasterfileData')
    def uploadMasterfileData(self, data):
        """
        Upload data to a masterfile
        """
        masterfile = self.getMasterfile()
        msg = masterfile.upload(data, self)

    def PUT(self, REQUEST, RESPONSE):
        """
        Handle HTTP PUT requests
        """
        #zLOG.LOG("%s: %s" % (self.meta_type, self.getMediaServer().meta_type), zLOG.INFO, 'Called PUT')
        self.dav__init(REQUEST, RESPONSE)
        #self.state = 'uploading'
        content_type = REQUEST.get_header('content-type', None)
        content_length = REQUEST.get_header('Content-Length', None)
        #fn = string.split(REQUEST['HTTP_CONTENT_DISPOSITION'],'=') # if we ever need the original filename (thomas)
        instream = REQUEST['BODYFILE']
        #zLOG.LOG("%s: %s" % (self.meta_type, self.getMediaServer().meta_type), zLOG.INFO, 'Before calling upload')
        resp = self.uploadMasterfileData(instream)
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'Resp is %s' % str(resp))

        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected(CMFCorePermissions.View, 'manage_FTPget')
    def manage_FTPget(self):
        "Get the document body for FTP download (also used for the WebDAV SRC)"
        iconid = self.getMediaServer().getIcon()
        return self.restrictedTraverse(iconid)

    def manage_FTPstat(self,REQUEST):
        "psuedo stat, used by FTP for directory listings"
        from AccessControl.User import nobody
        import marshal
        mode=0100000
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_FTPstat called')

        # check read permissions
        if (hasattr(aq_base(self),'manage_FTPget') and
            hasattr(self.manage_FTPget, '__roles__')):
            try:
                if getSecurityManager().validateValue(self.manage_FTPget):
                    mode=mode | 0440
            except: pass
            if nobody.allowed(self.manage_FTPget,
                              self.manage_FTPget.__roles__):
                mode=mode | 0004
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'mode after checking read permissions %s' % str(mode))

        # check write permissions
        if hasattr(aq_base(self),'PUT') and hasattr(self.PUT, '__roles__'):
            try:
                if getSecurityManager().validateValue(self.PUT):
                    mode=mode | 0220
            except: pass

            if nobody.allowed(self.PUT, self.PUT.__roles__):
                mode=mode | 0002

        mode = 0770
        #zLOG.LOG(self.meta_type, zLOG.INFO, 'mode after checking write permissions %s' % str(mode))

        # get size
        if hasattr(aq_base(self), 'get_size'):
            size=self.get_size()
        elif hasattr(aq_base(self),'manage_FTPget'):
            size=len(self.manage_FTPget())
        else:
            size=0
        # get modification time
        if hasattr(aq_base(self), 'bobobase_modification_time'):
            mtime=self.bobobase_modification_time().timeTime()
        else:
            mtime=time.time()
        # get owner and group
        owner=group='Zope'
        if hasattr(aq_base(self), 'get_local_roles'):
            for user, roles in self.get_local_roles():
                if 'Owner' in roles:
                    owner=user
                    break
        return marshal.dumps((mode,0,0,1,owner,group,size,mtime,mtime,mtime))

    security.declareProtected(CMFCorePermissions.View, 'getFTPURL')
    def getFTPURL(self):
        """
        Return an url to login via FTP
        """
        ms = getToolByName(self, 'portal_membership')
        member = ms.getAuthenticatedMember()
        username = member.getMemberId()
        pwd = member.getPassword()
        if pwd[:5] == '{SHA}' or pwd[:7] == '{CRYPT}':
            pwd = ''
        else:
            pwd = ':' + pwd
        port = os.getenv('FTP_PORT', '')
        server = self.REQUEST.get('SERVER_NAME', '')
        path = '/'+self.absolute_url(relative=1)
        vp = self.REQUEST.get('VirtualRootPhysicalPath', None)
        if vp: path = '/'+vp[1] + path
        #zLOG.LOG('Slide Show', zLOG.INFO, 'getFTPURL: \nserver: %s\nport: %s' % (server, port))
        if port:
            return "ftp://%s%s@%s:%s%s" % (username, pwd, server, port, path)
        else:
            return None

        """
        server = self.getMediaServer()
        user = server.getUploadLogin()
        passwd = server.getUploadPassword()
        if not passwd:
            passwd = ''
        else:
            passwd = ':' + passwd
        host = server.getHost()
        path = server.getUploadDir()

        return "ftp://%s%s@%s%s" % (user, passwd, host, path)
        """

    security.declareProtected(CMFCorePermissions.View, 'get_size')
    def get_size( self ):
        """ Used for FTP and apparently the ZMI now too """
        return 1

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setEncoderStart')
    def setEncoderStart(self, encstart=None):
        """
        Set the encoder start time
        """
        if encstart is None:
            encstart = DateTime()
        self._encoder_start = encstart

    security.declarePublic('getEncoderStart')
    def getEncoderStart(self):
        """
        Return the encoder start time
        """
        return self._encoder_start

    security.declarePublic('getMediaServer')
    def getMediaServer(self):
        """
        Return the media server corresponding to the serverid
        """
        medianet = getToolByName(self, 'portal_medianet')
        return medianet.getServerById(self._serverid)

    security.declarePublic('getInfoAttrs')
    def getInfoAttrs(self):
        server = self.getMediaServer()
        serverattrs = server.getDataListForServerInfo()

        for attrdict in serverattrs:
            #zLOG.LOG(self.meta_type, zLOG.INFO, 'attrdict 1: %s' % str(attrdict))
            id = attrdict['id']
            infoattr = getattr(self, '_%s' % id, None)
            #zLOG.LOG(self.meta_type, zLOG.INFO, 'infoattr: %s' % str(infoattr))
            if not infoattr:
                if attrdict['type'] in ('selection', 'multiple selection') and type(attrdict['options']) in (type(()), type([])):
                    #zLOG.LOG(self.meta_type, zLOG.INFO, 'Got a list: %s' % str(attrdict))
                    if len(attrdict['value']) == 1: # if there's only one choice pre-select it
                        #zLOG.LOG(self.meta_type, zLOG.INFO, 'setting %s to %s' % (id, infoattr))
                        infoattr = attrdict['options'][0]['value']
                        setattr(self, '_%s' % id, infoattr)
            if infoattr:
                attrdict['value']= infoattr
            #zLOG.LOG(self.meta_type, zLOG.INFO, 'attrdict 2: %s' % str(attrdict))
        attrlist = serverattrs
        return attrlist
